@@ -100,19 +100,20 @@ constraintExp2 <- function(p, mu, sigma = 1, k = 0.05, fixed=NULL) {
 #' optimization with the trust optimizer, see \link{mstrust}.
 #' @param data object of class \link{datalist}
 #' @param x object of class \link{prdfn}
-#' @param errmodel object of class \link{obsfn}
+#' @param errmodel object of class \link{obsfn}. \code{errmodel} does not need to be defined for all conditions.
 #' @param times numeric vector, additional time points where the prediction function is 
 #' evaluated. If NULL, time points are extacted from the datalist solely. If the prediction
 #' function makes use of events, hand over event \code{times} here.
 #' @param attr.name character. The constraint value is additionally returned in an 
 #' attributed with this name
+#' @param loq named numeric or single valued numeric. Limit of quantification.
 #' @return Object of class \code{obsfn}, i.e. a function 
 #' \code{obj(..., fixed, deriv, conditions, env)} that returns an objective list,
 #' \link{objlist}.
 #' @details Objective functions can be combined by the "+" operator, see \link{sumobjfn}.
 #' @example inst/examples/normL2.R
 #' @export
-normL2 <- function(data, x, errmodel = NULL, times = NULL, attr.name = "data") {
+normL2 <- function(data, x, errmodel = NULL, times = NULL, attr.name = "data", loq = -Inf) {
 
   timesD <- sort(unique(c(0, do.call(c, lapply(data, function(d) d$time)))))
   if (!is.null(times)) timesD <- sort(union(times, timesD))
@@ -121,8 +122,9 @@ normL2 <- function(data, x, errmodel = NULL, times = NULL, attr.name = "data") {
   data.conditions <- names(data)
   if (!all(data.conditions %in% x.conditions)) 
     stop("The prediction function does not provide predictions for all conditions in the data.")
+  e.conditions <- names(attr(errmodel, "mappings"))
   
-  controls <- list(times = timesD, attr.name = attr.name, conditions = x.conditions)
+  controls <- list(times = timesD, attr.name = attr.name, conditions = x.conditions, loq = loq)
 
   # might be necessary to "store" errmodel in the objective function (-> runbg)
   force(errmodel)  
@@ -144,6 +146,7 @@ normL2 <- function(data, x, errmodel = NULL, times = NULL, attr.name = "data") {
     # Import from controls
     timesD <- controls$times
     attr.name <- controls$attr.name
+    loq <- controls$loq
     
     # Create new environment if necessary
     if (is.null(env)) env <- new.env()
@@ -153,11 +156,11 @@ normL2 <- function(data, x, errmodel = NULL, times = NULL, attr.name = "data") {
     # Apply res() and wrss() to compute residuals and the weighted residual sum of squares
     out.data <- lapply(conditions, function(cn) {
       err <- NULL
-      if (!is.null(errmodel)) {
+      if ((!is.null(errmodel) & is.null(e.conditions)) | (!is.null(e.conditions) && (cn %in% e.conditions))) {
         err <- errmodel(out = prediction[[cn]], pars = getParameters(prediction[[cn]]), conditions = cn)
-        mywrss <- nll(res(data[[cn]], prediction[[cn]], err[[cn]]))
+        mywrss <- nll(res(data[[cn]], prediction[[cn]], err[[cn]], loq))
       } else {
-        mywrss <- wrss(res(data[[cn]], prediction[[cn]]))  
+        mywrss <- wrss(res(data[[cn]], prediction[[cn]], NULL, loq))  
       }
       available <- intersect(pars_out, names(mywrss$gradient))
       result <- template
@@ -316,9 +319,9 @@ constraintL2 <- function(mu, sigma = 1, attr.name = "prior", condition = NULL) {
       
       # Multiply with Jacobian of sigma vector if estimate sigma
       if (estimateSigma) {
+        val.sigma.sigma <- t(Jsigma) %*% val.sigma.sigma %*% Jsigma + diag((t(val.sigma) %*% Jsigma)[1,], ncol(Jsigma), ncol(Jsigma))
         val.sigma <- (val.sigma %*% Jsigma)[1,]
         val.p.sigma <- (val.p.sigma %*% Jsigma)
-        val.sigma.sigma <- t(Jsigma) %*% val.sigma.sigma %*% Jsigma
       }
       
       
@@ -608,24 +611,71 @@ priorL2 <- function(mu, lambda = "lambda", attr.name = "prior", condition = NULL
 #' @export
 wrss <- function(nout) {
   
+  # Extract BLOQ part from nout
+  is.bloq <- nout$bloq
+  nout.bloq <- nout[is.bloq, , drop = FALSE]
   
+  # Drop BLOQ part from nout
+  nout <- nout[!is.bloq, , drop = FALSE]
+  obj <- sum(nout$weighted.residual^2) + sum(-2*log(pnorm(-nout.bloq$weighted.residual)))
   
-  obj <- sum(nout$weighted.residual^2)
   grad <- NULL
   hessian <- NULL
-  
-  if (!is.null(attr(nout, "deriv"))) {
+  derivs <- attr(nout, "deriv")
+  if (!is.null(derivs)) {
+    
+    # Extract BLOQ part from derivs
+    derivs.bloq <- derivs[is.bloq, , drop = FALSE]
+    # Drop BLOQ part from derivs
+    derivs <- derivs[!is.bloq, , drop = FALSE]
+
+    if (nrow(derivs) > 0) {
+      
+      nout$sigma[is.na(nout$sigma)] <- 1 #replace by neutral element
+      sens <- as.matrix(derivs[, -(1:2), drop = FALSE])
+      
+      res <- nout$residual
+      sigma <- nout$sigma
+      
+      grad <- as.vector(2*matrix(res/sigma^2, nrow = 1) %*% sens)
+      names(grad) <- colnames(sens)
+      hessian <- 2*t(sens/sigma) %*% (sens/sigma)
+      
+    }
+    
+    if (nrow(derivs.bloq) > 0) {
+      
+      nout.bloq[is.na(nout.bloq$sigma)] <- 1
+      sens.bloq <- as.matrix(derivs.bloq[, -(1:2), drop = FALSE])
+      
+      Phi <- pnorm(-nout.bloq$weighted.residual)
+      G <- dnorm(-nout.bloq$weighted.residual)
+      res <- nout.bloq$residual
+      sigma <- nout.bloq$sigma
+      
+      
+      grad.bloq <- as.vector(matrix(2*G/(Phi*sigma), nrow = 1) %*% sens.bloq)
+      names(grad.bloq) <- colnames(sens.bloq)
+      
+      X1 <- sens.bloq*(G/(Phi*sigma))^2
+      X2 <- sens.bloq*(res*G/(Phi*sigma^3))
+      hessian.bloq <- 2 * t(X1) %*% sens.bloq - 2 * t(X2) %*% sens.bloq
+      
+      if (is.null(grad) & is.null(hessian)) {
+        grad <- grad.bloq
+        hessian <- hessian.bloq
+      } else {
+        grad <- grad + grad.bloq
+        hessian <- hessian + hessian.bloq
+      }
+      
+    }
+    
     
-    nout$sigma[is.na(nout$sigma)] <- 1 #replace by neutral element
-  
-    sens <- as.matrix(attr(nout, "deriv")[, -(1:2), drop = FALSE])
-    grad <- as.vector(2*matrix(nout$residual/nout$sigma^2, nrow = 1) %*% sens)
-    names(grad) <- colnames(sens)
-    hessian <- 2*t(sens/nout$sigma) %*% (sens/nout$sigma)
     
   }
   
-  
+
   objlist(value = obj, gradient = grad, hessian = hessian)
   
 }
@@ -636,25 +686,91 @@ wrss <- function(nout) {
 #' gradient (numeric, gradient) and 
 #' hessian (matrix of type numeric).
 #' @export
+#' @importFrom stats pnorm dnorm
 nll <- function(nout) {
   
-  obj <- sum(nout$weighted.residual^2) + 2 * sum(log(nout$sigma))
+  # Extract BLOQ part from nout
+  is.bloq <- nout$bloq
+  nout.bloq <- nout[is.bloq, , drop = FALSE]
+  
+  # Drop BLOQ part from nout
+  nout <- nout[!is.bloq, , drop = FALSE]
+  
+  obj <- sum(nout$weighted.residual^2) + sum(log(2*pi*nout$sigma^2)) + 
+    sum(-2*log(pnorm(-nout.bloq$weighted.residual)))
   grad <- NULL
   hessian <- NULL
   
   
-  if (!is.null(attr(nout, "deriv")) & !is.null(attr(nout, "deriv.err"))) {
+  
+  derivs <- attr(nout, "deriv")
+  derivs.err <- attr(nout, "deriv.err")
+  if (!is.null(derivs) & !is.null(derivs.err)) {
     
+    # Extract BLOQ part from derivs
+    derivs.bloq <- derivs[is.bloq, , drop = FALSE]
+    derivs.err.bloq <- derivs.err[is.bloq, , drop = FALSE]
+    # Drop BLOQ part from derivs
+    derivs <- derivs[!is.bloq, , drop = FALSE]
+    derivs.err <- derivs.err[!is.bloq, , drop = FALSE]
     
-    sens <- as.matrix(attr(nout, "deriv")[, -(1:2), drop = FALSE])
-    sens.err <- as.matrix(attr(nout, "deriv.err")[, -(1:2), drop = FALSE])
+    if (nrow(derivs) > 0 & nrow(derivs.err) > 0) {
     
-    grad <- as.vector(2*matrix(nout$residual/nout$sigma^2, nrow = 1) %*% sens -
-                        2*matrix(nout$residual^2/nout$sigma^3, nrow = 1) %*% sens.err +
-                        2*matrix(1/nout$sigma, nrow = 1) %*% sens.err)
-    names(grad) <- colnames(sens)
+      # Get sensitivities: sens = dres/dp, sens.err = dsigma/dp
+      sens <- as.matrix(derivs[, -(1:2), drop = FALSE])
+      sens.err <- as.matrix(derivs.err[, -(1:2), drop = FALSE])
+      
+      res <- nout$residual
+      sigma <- nout$sigma
+      
+      # Compute gradient
+      grad <- as.vector(2*matrix(res/sigma^2, nrow = 1) %*% sens -
+                          2*matrix(res^2/sigma^3, nrow = 1) %*% sens.err +
+                          2*matrix(1/sigma, nrow = 1) %*% sens.err)
+      names(grad) <- colnames(sens)
+      
+      # Compute hessian
+      X1 <- (1/sigma)*sens - (res/sigma^2)*sens.err
+      X2 <- (res/sigma^2)*sens.err
+      X3 <- (1/sigma)*sens.err
+      
+      hessian <- 2 * t(X1) %*% X1 #+ 4 * t(X2) %*% X2 - 2 * t(X3) %*% X3
+      
+    }
+    
+    if (nrow(derivs.bloq) > 0 & nrow(derivs.err.bloq) > 0) {
+      
+      # Get sensitivities: sens = dres/dp, sens.err = dsigma/dp
+      sens.bloq <- as.matrix(derivs.bloq[, -(1:2), drop = FALSE])
+      sens.err.bloq <- as.matrix(derivs.err.bloq[, -(1:2), drop = FALSE])
+      
+      Phi <- pnorm(-nout.bloq$weighted.residual)
+      G <- dnorm(-nout.bloq$weighted.residual)
+      res <- nout.bloq$residual
+      sigma <- nout.bloq$sigma
+      
+      # Compute gradient
+      grad.bloq <- as.vector(matrix(2*G/(Phi*sigma), nrow = 1) %*% sens.bloq) -
+        as.vector(matrix(2*G*res/(Phi*sigma^2), nrow = 1) %*% sens.err.bloq)
+      names(grad.bloq) <- colnames(sens.bloq)
+      
+      # Compute hessian
+      X <- -(1/sigma)*sens.bloq + (res/sigma^2)*sens.err.bloq
+      X1 <- (2*G^2/Phi^2)*X
+      X2 <- (2*G*res/(Phi*sigma))*X
+      
+      hessian.bloq <- t(X1) %*% X  - t(X2) %*% X
+      
+      if (is.null(grad) & is.null(hessian)) {
+        grad <- grad.bloq
+        hessian <- hessian.bloq
+      } else {
+        grad <- grad + grad.bloq
+        hessian <- hessian + hessian.bloq
+      }
+      
+    }
     
-    hessian <- 2 * t(sens/nout$sigma - sens.err*nout$residual/nout$sigma^2) %*% (sens/nout$sigma - sens.err*nout$residual/nout$sigma^2) # - 2 * t(sens.err/nout$sigma) %*% (sens.err/nout$sigma)
     
   }
   
