@@ -1,8 +1,13 @@
+## Functions to generate parameter transformation ----
+
+
 #' Generate a parameter transformation function
 #' @description  Generate parameter transformation function from a
 #' named character vector or object of class \link{eqnvec}. This is a wrapper
 #' function for \link{Pexpl} and \link{Pimpl}. See for more details there.
-#' @param trafo object of class \code{eqnvec} or named character
+#' @param trafo object of class \code{eqnvec} or named character or list thereof. In case,
+#' trafo is a list, \code{P()} is called on each element and conditions are assumed to be
+#' the list names.
 #' @param parameters character vector
 #' @param condition character, the condition for which the transformation is generated
 #' @param attach.input attach those incoming parameters to output which are not overwritten by
@@ -10,20 +15,35 @@
 #' @param keep.root logical, applies for \code{method = "implicit"}. The root of the last
 #' evaluation of the parameter transformation function is saved as guess for the next 
 #' evaluation.
-#' @param compile logical
-#' @param modelname character
+#' @param compile logical, compile the function (see \link{funC0})
+#' @param modelname character, see \link{funC0}
 #' @param method character, either \code{"explicit"} or \code{"implicit"}
 #' @param verbose Print out information during compilation
 #' @return An object of class \link{parfn}.
+#' @importFrom digest digest
 #' @export
 P <- function(trafo = NULL, parameters=NULL, condition = NULL, attach.input = FALSE,  keep.root = TRUE, compile = FALSE, modelname = NULL, method = c("explicit", "implicit"), verbose = FALSE) {
   
   if (is.null(trafo)) return()
+  if (!is.list(trafo)) {
+    trafo <- list(trafo)
+    names(trafo) <- condition
+  }
   
   method <- match.arg(method)
-  switch(method, 
-         explicit = Pexpl(trafo = as.eqnvec(trafo), parameters = parameters, attach.input = attach.input, condition = condition, compile = compile, modelname = modelname, verbose = verbose),
-         implicit = Pimpl(trafo = as.eqnvec(trafo), parameters = parameters, keep.root = keep.root, condition = condition, compile = compile, modelname = modelname, verbose = verbose))
+  
+  # Add hash to prevent overwriting of .c-files with different content
+  if (!is.null(modelname)) modelname <- paste0(modelname, "_", substr(digest(list(trafo, parameters)),1,8))
+
+  Reduce("+", lapply(1:length(trafo), function(i) {
+  
+      switch(method, 
+           explicit = Pexpl(trafo = as.eqnvec(trafo[[i]]), parameters = parameters, attach.input = attach.input, condition = names(trafo[i]), compile = compile, modelname = modelname, verbose = verbose),
+           implicit = Pimpl(trafo = as.eqnvec(trafo[[i]]), parameters = parameters, keep.root = keep.root, condition = names(trafo[i]), compile = compile, modelname = modelname, verbose = verbose))
+    
+    
+  }))
+  
   
 }
 
@@ -339,3 +359,244 @@ Pimpl <- function(trafo, parameters=NULL, condition = NULL, keep.root = TRUE, po
   
 }
 
+
+
+
+## Functions to simplify the creation of parameter transformations ----
+
+#' Define parameter transformations by \code{define()}, \code{branch()} and \code{insert()}
+#' 
+#' @param trafo named character vector of parametric expressions or object 
+#' of class \code{eqnvec}
+#' @param expr character of the form \code{"lhs ~ rhs"} where both \code{lhs}
+#' and \code{rhs} can contain a number of symbols for which vaues are passed
+#' by the \code{...} argument
+#' @param  conditionMatch optional character, Use as regular expression to apply the reparameterization only to conditions containing conditionMatch
+#' @param ... used to pass values for symbols as named arguments
+#' @return object of the same class as trafo or list thereof, if \code{branch()} has been
+#' used.
+#' @export
+#' @example inst/examples/define.R
+define <- function(trafo, expr, ..., conditionMatch = NULL) {
+  
+  if (missing(trafo)) trafo <- NULL
+  lookuptable <- attr(trafo, "tree")
+  
+  
+  if (is.list(trafo) & is.null(names(trafo)))
+    stop("If trafo is a list, elements must be named.")
+  
+  if (is.list(trafo) & !all(names(trafo) %in% rownames(lookuptable)))
+    stop("If trafo is a list and contains a lookuptable (is branched from a tree), the list names must be contained in the rownames of the tree.")
+  
+  if (!is.list(trafo)) {
+    mytrafo <- list(trafo)
+  } else {
+    mytrafo <- trafo
+  }
+  
+  dots <- substitute(alist(...))
+  out <- lapply(1:length(mytrafo), function(i) {
+    
+    .currentTrafo <- mytrafo[[i]]
+    .currentSymbols <- NULL
+    if (!is.null(.currentTrafo))
+      .currentSymbols <- getSymbols(.currentTrafo)
+    
+    if (is.list(trafo)) {
+      mytable <- lookuptable[names(mytrafo)[i], , drop = FALSE]
+    } else {
+      mytable <- lookuptable[1, , drop = FALSE]
+    }
+    
+    if ((!is.null(conditionMatch)))
+      if((!str_detect(rownames(mytable), conditionMatch))) 
+        return(mytrafo[[i]])
+    
+    with(mytable, {
+      args <- c(list(expr = expr, trafo = mytrafo[[i]], reset = TRUE), eval(dots))
+      do.call(repar, args)
+    })
+    
+    
+  })
+  names(out) <- names(mytrafo)
+  if (!is.list(trafo)) out <- out[[1]]
+  attr(out, "tree") <- lookuptable
+  
+  return(out)
+  
+}
+
+
+#' @export
+#' @rdname define
+insert <- function(trafo, expr, ..., conditionMatch = NULL) {
+  
+  
+  if (missing(trafo)) trafo <- NULL
+  lookuptable <- attr(trafo, "tree")
+  
+  
+  if (is.list(trafo) & is.null(names(trafo)))
+    stop("If trafo is a list, elements must be named.")
+  
+  if (is.list(trafo) & !all(names(trafo) %in% rownames(lookuptable)))
+    stop("If trafo is a list and contains a lookuptable (is branched from a tree), the list names must be contained in the rownames of the tree.")
+  
+  if (!is.list(trafo)) {
+    mytrafo <- list(trafo)
+  } else {
+    mytrafo <- trafo
+  }
+  
+  dots <- substitute(alist(...))
+  out <- lapply(1:length(mytrafo), function(i) {
+    
+    .currentTrafo <- mytrafo[[i]]
+    .currentSymbols <- NULL
+    if (!is.null(.currentTrafo))
+      .currentSymbols <- getSymbols(.currentTrafo)
+    
+    if (is.list(trafo)) {
+      mytable <- lookuptable[names(mytrafo)[i], , drop = FALSE]
+    } else {
+      mytable <- lookuptable[1, , drop = FALSE]
+    }
+    
+    if ((!is.null(conditionMatch)))
+      if((!str_detect(rownames(mytable), conditionMatch))) 
+        return(.currentTrafo)
+    
+    
+    
+    with(mytable, {
+      .fun <- function() {
+        # subset conditions by logicals expressions supplied by the dots
+        dots_eval <- eval(dots)                                            # convert from substituted to language
+        if (length(dots_eval) == 0) 
+          return(do.call(repar, list(expr = expr, trafo = .currentTrafo)))
+        
+        dots_eval_eval <- lapply(dots_eval, function(i) eval.parent(i, 3)) # evaluate the language in the "mytable" frame. parent1: lapply, parent2: .fun, parent3: with
+        which_logical <- vapply(dots_eval_eval, function(i) {is.logical(i)}, FUN.VALUE = vector("logical", 1)) # which of the dots are logical
+        logical_dots <- dots_eval[which_logical]                           # subset to matching conditions
+        matching <- do.call(c, logical_dots)
+        if(!is.null(matching)) { # null means no logical dots were supplied
+          if (any(!matching)) {  
+            return(.currentTrafo) }}
+        
+        args <- c(list(expr = expr, trafo = .currentTrafo), dots_eval_eval[!which_logical]) # feed the rest of the eval'd dots into repar
+        return(do.call(repar, args))
+      }
+      .fun()
+    })
+    
+    
+  })
+  names(out) <- names(mytrafo)
+  if (!is.list(trafo)) out <- out[[1]]
+  attr(out, "tree") <- lookuptable
+  
+  return(out)
+  
+}
+
+
+
+
+
+#' @export
+#' @rdname define
+#' @param table table of covariates as data frame. Rownames are used as unique identifier,
+#' usually called "conditions", and columns represent covariates associated with these conditions.
+#' @param conditions character vector with condition names. Overwrites the rownames of table.
+branch <- function(trafo, table = NULL, conditions = rownames(table)) {
+  
+  if (is.null(table) & is.null(conditions)) 
+    return(trafo)
+  
+  if (is.null(conditions)) conditions <- paste0("C", 1:nrow(table))
+  if (is.null(table)) table <- data.frame(condition = conditions, row.names = conditions)
+  
+  out <- lapply(conditions, function(x) trafo)
+  names(out) <- conditions
+  
+  rownames(table) <- conditions
+  attr(out, "tree") <- table
+  
+  return(out)  
+  
+}
+
+
+#' Reparameterization
+#' 
+#' @param expr character of the form \code{"lhs ~ rhs"} where \code{rhs}
+#' reparameterizes \code{lhs}. Both \code{lhs} and \code{rhs}
+#' can contain a number of symbols whose values need to be passed by the \code{...} argument.
+#' @param trafo character or equation vector or list thereof. The object where the replacement takes place in
+#' @param ... pass symbols as named arguments
+#' @param reset logical. If true, the trafo element corresponding to lhs is reset according to rhs. 
+#' If false, lhs wherever it occurs in the rhs of trafo is replaced by rhs of the formula.
+#' @return an equation vector with the reparameterization.
+#' @details Left and right-hand side of \code{expr} are searched for symbols. If separated by
+#' "_", symbols are recognized as such, e.g. in \code{Delta_x} where the symbols are 
+#' "Delta" and "x". Each symbol for which values (character or numbers) are passed by the
+#' \code{...} argument is replaced.
+#' @export
+#' @importFrom stats as.formula
+#' @examples
+#' innerpars <- letters[1:3]
+#' constraints <- c(a = "b + c")
+#' mycondition <- "cond1"
+#' 
+#' trafo <- repar("x ~ x", x = innerpars)
+#' trafo <- repar("x ~ y", trafo, x = names(constraints), y = constraints)
+#' trafo <- repar("x ~ exp(x)", trafo, x = innerpars)
+#' trafo <- repar("x ~ x + Delta_x_condition", trafo, x = innerpars, condition = mycondition)
+repar <- function(expr, trafo = NULL, ..., reset = FALSE) {
+  
+  if (inherits(expr, "formula")) expr <- deparse(expr)
+  
+  parsed.expr <- as.character(stats::as.formula(gsub("_", ":", expr, fixed = TRUE)))
+  lhs <- parsed.expr[2]
+  lhs.symbols <- getSymbols(lhs)
+  rhs <- parsed.expr[3]
+  rhs.symbols <- getSymbols(rhs)
+  
+  # Make sure that arguments are characters
+  args <- lapply(list(...), as.character)
+  
+  replacements <- as.data.frame(args, stringsAsFactors = FALSE)
+  
+  lhs <- sapply(1:nrow(replacements), function(i) {
+    out <- replaceSymbols(colnames(replacements), replacements[i, ], lhs)
+    gsub(":", "_", out, fixed = TRUE)
+  })
+  
+  rhs <- sapply(1:nrow(replacements), function(i) {
+    out <- replaceSymbols(colnames(replacements), replacements[i, ], rhs)
+    gsub(":", "_", out, fixed = TRUE)
+  })
+  
+  if (is.null(trafo)) {
+    trafo <- as.eqnvec(structure(lhs, names = lhs))
+  } else if (is.list(trafo) & !reset) {
+    trafo <- lapply(trafo, function(t) replaceSymbols(lhs, rhs, t))
+  } else if (is.character(trafo) & !reset) {
+    trafo <- replaceSymbols(lhs, rhs, trafo)
+  } else if (is.list(trafo) & reset) {
+    trafo <- lapply(trafo, function(t) {t[lhs] <- rhs; return(t)})
+  } else if (is.character(trafo) & reset) {
+    trafo[lhs] <- rhs
+  }
+  
+  return(trafo)
+  
+  
+}
+
+paste_ <- function(...) paste(..., sep = "_")
+
+
+
